using System;
using System.Security ;
using System.Security.Principal  ;
using System.Collections ;
using System.Xml;
using System.Reflection;
using System.IO;
using System.Threading;
using System.Text ;

namespace CSTSecurity
{

	public interface IOperationCheck {
		bool IsOperationAllowed(string operation, params string[] p_params);
	}
	public class AuthenticationModuleEx
	{
		public AuthenticationModuleEx()
		{
		}
		public static void Login() 
		{
			MycustomPrincipal l_MycustomPrincipal = new  
				MycustomPrincipal (WindowsIdentity.GetCurrent());
			Thread.CurrentPrincipal = l_MycustomPrincipal ;
			//this fails if set more than once
			try 
			{
				AppDomain.CurrentDomain.SetThreadPrincipal(l_MycustomPrincipal);
			}
			catch {}
			}
		public class MycustomPrincipal : IPrincipal		, IOperationCheck
		{
			private static XmlDocument  i_xmlroles= new XmlDocument ();
			private static XmlDocument  i_xmloperations= new XmlDocument ();

			private IIdentity i_Identity =null;
			private ArrayList i_roles=new ArrayList() ;

			static MycustomPrincipal() 
			{
				i_xmlroles.Load (Path.GetDirectoryName (Assembly.GetExecutingAssembly().Location)  + @"\Roles.xml" );
				i_xmloperations.Load (Path.GetDirectoryName (Assembly.GetExecutingAssembly().Location)  + @"\Operations.xml" );
			}

			//code outside this assembly can't create this principal object
			internal MycustomPrincipal(IIdentity p_IIdentity) 
			{
				lock (i_xmlroles) 
				{
					XmlNodeList l_roles =  i_xmlroles.SelectNodes ("//role[user/@name='" + p_IIdentity.Name  + "']");
					foreach (XmlNode l_rl in l_roles ) 
						i_roles.Add (l_rl.Attributes.GetNamedItem ("name").Value );						
				}
			}
			public IIdentity Identity
			{
				get 
				{
					return i_Identity;
				}
			}

			public bool IsInRole(string role)
			{
				return i_roles.Contains (role);
			}

			public bool IsOperationAllowed(string operation, params string[] p_params)
			{
				lock (i_xmloperations) {
					XmlNodeList l_nodelist = i_xmloperations.SelectNodes("/root/operation[@name='" +
						operation + "'" + pf_buildRoleClause(p_params) + pf_buildParamClause(p_params) + "]");
					return l_nodelist.Count > 0 ? true: false;
				}
			}

			private string pf_buildParamClause(string[] p_params) {
				if (p_params.Length == 0 ) return "";
				StringBuilder l_sb = new StringBuilder ();
				foreach(string p_param in p_params ) {
					string[] l_param = p_param.Split ('=');
					if (l_param.Length !=2)
						throw new Exception ("invalid Format");
					l_sb.Append (" and ");					
					l_sb.Append ("params/param/@name='" + l_param[0] + "' and params/param/@value='" + l_param[1] + "'") ;					
				}
				return l_sb.ToString ();
			}

			private string pf_buildRoleClause(string[] p_params) 
			{
				if (i_roles.Count == 0 ) return "";
				StringBuilder l_sb = new StringBuilder ();
				l_sb.Append (" and (");					
				for(int i=0; i < i_roles.Count ; i++) 
				{
					if (p_params.Length ==0)
						l_sb.Append ("role/@name='" + i_roles[i] + "'") ;					
					else
						l_sb.Append ("params/role/@name='" + i_roles[i] + "'") ;					
					if(i!= (i_roles.Count -1))
						l_sb.Append (" or ") ;					
				}
				l_sb.Append (")");					
				return l_sb.ToString ();
			}
		
		}
	}
}
